function [Fballoc, Fscfi, Fscf, Fsb, fs, bitrate, sblimit] = MP2read(show, Nbytes)
% Function for reading simple MP2 frame from disc, coded binary

global BArow BAQuantNo Qbits Qgroup
 
  Fballoc=[]; Fscfi=[]; Fscf=[]; Fsb=[];  % cleanup
  
% Read the whole file  
  FileIN = fopen('frame.mpg','r');
  Frame = fread(FileIN,inf,'uchar'); 
  fclose(FileIN);
  if(show==1) Frame, pause; end
  
% Take only first Nbytes to char table, since there is no frame synchronization 
  FrameS = '';
  for(k = 1 : Nbytes) FrameS = strcat( FrameS, dec2bin(Frame(k),8)); end
  
  if (show==1)
     disp( sprintf('Frame         = %s', FrameS( 1:11)) );
     disp( sprintf('ID            = %s', FrameS(12:13)) );
     disp( sprintf('Layer         = %s', FrameS(14:15)) );
     disp( sprintf('Bitsream      = %s', FrameS(17:20)) );
     disp( sprintf('Frequency     = %s', FrameS(21:22)) );
     disp( sprintf('Stereo/mono   = %s', FrameS(25:26)) );
     pause
  end
  
  Nbitrate = bin2dec( FrameS(17:20) );  % code for bitstream
  Nfs = bin2dec( FrameS(21:22) );       % code for sampling frequency
  
  switch( Nbitrate )                    % bitrate
  case 4,  bitrate = 64;                % 64 kbs (kilobits per second) 
  case 6,  bitrate = 96;                % 96
  case 8,  bitrate = 128;               % 128
  case 10, bitrate = 192;               % 192
  otherwise, disp('WRONG input bitsream code'); pause; return;
  end
  
  switch( Nfs )                         % czstotliwo prbkowania
  case 0, fs = 44100;                   % 44.1  kHz
  case 1, fs = 48000;                   % 48
  case 2, fs = 32000;                   % 32
  otherwise, disp('Za czstotliwo prbkowania'); pause; return;
  end
  
  sblimit=27; sb1=11; sb2=12; sb3=4;     % parametry "dostpu" do tablicy alokacji bitw    
  if ((fs==32000 || fs==48000) && (bitrate>80)) sblimit=30; sb3=7; end
  
  if(show==1) bitrate, fs, sblimit, pause; end 
  
% Bit allocation
  n1st=32;           Nb=4;  for n = 0:sb1-1,  Fballoc(n+1)         = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); end
  n1st=n1st+sb1*Nb;  Nb=3;  for n = 0:sb2-1,  Fballoc(sb1+n+1)     = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); end
  n1st=n1st+sb2*Nb;  Nb=2;  for n = 0:sb3-1,  Fballoc(sb1+sb2+n+1) = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); end
  n1st=n1st+sb3*Nb;
  if(show==1) Fballoc, pause; end
  
% Scale factor information  
  Nscfi = sum( Fballoc > 0 );
  Nb=2; for n = 0:Nscfi-1, Fscfi(n+1) = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); end
  n1st=n1st+Nscfi*Nb;
  if(show==1) Fscfi, pause; end
  
% Scale factors  
  Nscf = 3*sum(Fscfi==0) + 2*(sum(Fscfi==1)+sum(Fscfi==3)) + 1*sum(Fscfi==2);
  Nb=6; for n = 0:Nscf-1, Fscf(n+1) = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); end
  n1st=n1st+Nscf*Nb;
  if(show==1) Fscf, pause; end
  
% Audio samples
  m = 1; K = 12;
  for l = 1 : K % K=12 times
      for k = 1 : sblimit % 3 samples at one in each sub-band
          indeks = Fballoc(k);
          if(indeks > 0 )
             nrkwant = BAQuantNo( BArow(k), indeks );
             grupa   = Qgroup( nrkwant );
             Nb      = Qbits( nrkwant );
             if( grupa==3 ) % one code for three samples
                Fsb(m) = bin2dec( FrameS(n1st+1 : n1st+Nb) ); m=m+1;
                n1st = n1st+Nb;
             else  % one code per sample, three samples
                for n = 0:2, Fsb(m) = bin2dec( FrameS(n1st+1+n*Nb : n1st+Nb+n*Nb) ); m=m+1; end
                n1st = n1st+3*Nb;
             end    
          end
      end
  end     
  if(show==1) Fsb, pause; end
  
  Fsb = Fsb';
  