function MP2write( Fballoc, Fscfi, Fscf, Fsb, Nbytes, sblevels, sbbits, fs, bitsream, sblimit, show )
% Function for storing to disc MP2 data frame in binary format

persistent InitPrefix Prefix sblimit1 sblimit2 sblimit3;

  if( isempty(InitPrefix) )           % create header
     InitPrefix = 1;
     switch( bitsream )               % bitsream (compession ratio)
          case 64,  codestr='0100';   %  64 kbs (kilo bits per second)
          case 96,  codestr='0110';   %  96
          case 128, codestr='1000';   % 128
          case 192, codestr='0110';   % 192
          otherwise, disp('WRONG bitsream'); pause; return;
     end
     switch( fs )                    % samplinf requency
          case 44100,  codefs='00';  % 44.1 kHz 
          case 48000,  codefs='01';  % 48
          case 32000,  codefs='10';  % 32
          otherwise, disp('WRONG frequency'); pause; return;
     end
     
     Prefix = '';                              %(xx) = number of bits
     Prefix = strcat( Prefix, '11111111111');  %(11) synchronisation preambule (11 bits)
     Prefix = strcat( Prefix, '11');           %(2)  compression algorithm: 00=MPEG2.5, 10=MPEG2, 11=MPEG1
     Prefix = strcat( Prefix, '10');           %(2)  layer: 11=1, 10=2, 01=3, 00=reserved
     Prefix = strcat( Prefix, '1');            %(1)  CRC coding (error checking): 1=no, 0=yes
     Prefix = strcat( Prefix, codestr);        %(4)  bitstream: 0101=64, 0111=96, 1001=128, 1011=192
     Prefix = strcat( Prefix, codefs);         %(2)  sampling freq.: 00=44.1, 01=48, 10=32, 00=reserved
     Prefix = strcat( Prefix, '0');            %(1)  marker of extra byte: 0=no, 1=yes
     Prefix = strcat( Prefix, '0');            %(1)  free bit
     Prefix = strcat( Prefix, '11');           %(2)  mono/stereo: 00=stereo, 01=joint stereo, 10=stereo, 11=mono
     Prefix = strcat( Prefix, '00');           %(2)  only for joint stereo
     Prefix = strcat( Prefix, '0');            %(1)  copyright: 0=no, 1=yes
     Prefix = strcat( Prefix, '1');            %(1)  1=original, 0=copy
     Prefix = strcat( Prefix, '00');           %(2)  de-emfaza: 00=no, 01=50/15 mikrosec, 10=reserved, 11=CCITT J.17
     
     sblimit1=11; sblimit2=23; sblimit3=27;    %     number of rows in bit allocation table
     if ((fs==32000 || fs==48000) && (bitsream>80)) sblimit3=30; end
  end
% Prefix, pause
  
% Bit allocation
  BitAlloc = '';
  Nb=4; mod=2^Nb; for(k =          1 : sblimit1)  BitAlloc = strcat( BitAlloc, dec2bin( rem(Fballoc(k),mod), Nb )); end
  Nb=3; mod=2^Nb; for(k = sblimit1+1 : sblimit2)  BitAlloc = strcat( BitAlloc, dec2bin( rem(Fballoc(k),mod), Nb )); end
  Nb=2; mod=2^Nb; for(k = sblimit2+1 : sblimit3)  BitAlloc = strcat( BitAlloc, dec2bin( rem(Fballoc(k),mod), Nb )); end
  if(show==1) BitAlloc, pause; end
  
% Scale factor information (SFinfo) and indexes of scale factors (SF)
  SFinfo = ''; SF = '';
  Nb=2; mod=2^Nb; for(k=1:length(Fscfi))  SFinfo = strcat( SFinfo, dec2bin( rem(Fscfi(k),mod), Nb )); end
  Nb=6; mod=2^Nb; for(k=1:length(Fscf))   SF = strcat( SF, dec2bin( rem(Fscf(k),mod), Nb )); end 
  if(show==1) SFinfo, SF, pause; end
  
% Coding of audio samples in frequency sub-bands
  Samples = ''; LenAll=0;
  for m = 1 : 12 % K=12 times
      for k=1:sblimit % 3 samples at once in each sub-band
          k; Nb = sbbits(k); mod = 2^Nb;
          if (Nb ~= 0 )
              lev = sblevels(k); if( lev==3 || lev == 5 || lev ==9) many = 1; else many=3; end
              for m=1:many % 1 joint code or 3 codes in one sub-band
                  str = dec2bin( rem(Fsb(m),mod), Nb);
                  Samples = strcat( Samples, str );
              end   
              Fsb = Fsb(many+1:end); % remove already read information
            % Samples, pause
            % Nb, many, LenSamples = many*Nb, LenAll = LenAll + LenSamples, pause
          end
       end
  end    
  if(show==1) Samples, pause; end
   
  Frame = strcat( Prefix, BitAlloc, SFinfo, SF, Samples);
   
% Appending frame with zeros to the length Nbytes*8
    for k = length( Frame )+1 : Nbytes*8
        Frame = strcat( Frame, '0'); 
    end
   
% Show the frame
% disp('number of bytes'); Nbytes, length(Frame), pause
% disp('number of bits'); length(Frame), pause
% Frame, pause
   
% Writing the frome to separate file "frame.mpg" and appending it to the file "recording.mpg"
  FileOUT1 = fopen('frame.mpg','w');       % 'w' = write to file, remove old content
  FileOUT2 = fopen('recording.mpg','a');   % 'a' = append binary to the file end
 Nbytes = length(Frame)/8; 
  for k = 0 :Nbytes-1
      fwrite(FileOUT1, bin2dec( Frame(1+k*8 : 8+k*8) ),'uchar'); 
      fwrite(FileOUT2, bin2dec( Frame(1+k*8 : 8+k*8) ),'uchar'); 
  end      
  fclose(FileOUT1);
  fclose(FileOUT2);
  
if(0) % Test of writing/reading a single frame  
% ======================================================================   
% Reading  
  ZbiorIN = fopen('frame.mpg','r');
  FrameIN = '';
  for k = 1 : Nbytes
     FrameIN = strcat( FrameIN, dec2bin( fread(ZbiorIN,1,'uchar'), 8 ));
  end   
  fclose(ZbiorIN);
  Frame, FrameIN, pause % final comparison
  max( max( Frame - FrameIN)), pause
end
