% lab15_ex_aac.m
% Principles of AAC coding using sliding MDCT transform
clear all; close all;

Nmany = 100;                % number of audio frames
N = 2048;                   % window length
M = N/2;                    % window shift 
Nx = N+M*(Nmany-1);         % number of signal samples

% Input signal
[ x, fs ] = audioread('bach44100.wav'); size(x),   % music
x = x(1:Nx,1); x=x.';                              % cut, transpose
soundsc(x,fs);                                     % play
figure; plot(x); title('x(n)'); pause              % show

% MDCT and IMDCT transformation matrices
[n,k] = meshgrid(0:(N-1),0:(N/2-1));               % indexes
win = sin(pi*(n+0.5)/N);                           % window
C = sqrt(2/M)*win.*cos(pi/M*(k+1/2).*(n+1/2+M/2)); % MDCT matrix with window
D = C';                                            % IMDCT matrix with window

% Bit allocation for subbands: 2^b+1 quantization levels 
b = [5*ones(M/4,1); 4*ones(M/4,1); 3*ones(M/4,1); 2*ones(M/4,1)]; sc=2.^(b-1)+0.49; sc=sc';
%b = 5*ones(M,1); sc = 2.^(b-1)+0.49; sc=sc';

% AAC analysis: signal decomposition into sub-bands
y = zeros(1,Nx); sb = zeros(Nmany,M); figure;   % output signal, subbands
for k=1:Nmany                                   % analysis loop
    n1st = 1+(k-1)*M; nlast = N + (k-1)*M;      % next signal sample indexes
    n = n1st : nlast;                           % samples from-to
    bx = x( n );                                % without window
    BX = C*bx';                                 % MDCT with window in matrix C
    sb(k,1:M) = BX';                            % storing subbands for quantization
  % stem(BX); title('Samples in bands'); pause  % plot of sub-band samples
  % BX(N/4+1:N/2,1) = zeros(N/4,1);             % doing some processing
end

% Quantization in sub-bands
sbmax = max( abs(sb) );                         % finding max values
sb = sbmax .* fix( sc .* (sb./sbmax) ) ./ sc;   % sb quantization

% Signal synthesis: signal reconstruction from sub-band samples
for k=1:Nmany
    n1st = 1+(k-1)*M; nlast = N + (k-1)*M;      % next signal sample indexes
    n = n1st : nlast;                           % samples from-to
    BX = sb(k,1:M)';                            % restoring subbands
    by = D*BX;                                  % IMDCT with window in matrix D
    y( n ) = y( n ) + by';                      % restoring signal without window
end                                             %
xqs = y; clear y;

% Quantization of the original signal: 2^b+1 quantization levels
b=5; sc=2^(b-1)+0.49; xmax=max(abs(x)); xq = xmax * fix( sc*x/xmax ) / sc;

% Comparison
m=M+1:Nx-M;
figure; plot(m,x(m),'ro',m,xqs(m),'g*',m,xq(m),'bx'); title('x(R), xq(G), xqs(B)'); pause
SNR1 = 10*log10( sum(x(m).^2) / sum( (x(m)-xq(m)).^2 )  ),
SNR2 = 10*log10( sum(x(m).^2) / sum( (x(m)-xqs(m)).^2 ) ),
max_abs_error_xq  = max(abs(x(m)-xq(m))),
max_abs_error_xqs = max(abs(x(m)-xqs(m))),
[X  ,f] = periodogram(  x,[],512,fs,'power'); X  =10*log10(X);
[Xq ,f] = periodogram( xq,[],512,fs,'power'); Xq =10*log10(Xq);
[Xqs,f] = periodogram(xqs,[],512,fs,'power'); Xqs=10*log10(Xqs);
figure; plot(f,X,'r-',f,Xq,'b-',f,Xqs,'g-'); xlabel('f (Hz)'); title('Power (dB)');
legend('x(n)','xq(n)','xqs(n)','Location','SouthWest'); grid; pause
