% lab15_ex_filterbank.m
% MP2 filter bank - regular version

clear all; close all;

Nmany = 2000;                 % number of audio frames
N = 512;                      % window/filter length
M = 32;                       % number of sub-bands/channels 

% Test signal
Nx = N+M*(Nmany-1);           % number of signal samples
num = [ 2, 14 ];              % number of freq channels with signal components
fs = 44100;                   % sampling frequency
df = (fs/2)/M;                % sub-band width
fcentr = (0:M-1)*df + df/2;   % sub-band central frequencies
fc = fcentr(num);             % signal component frequencies

x = zeros(1,Nx);              % signal samples initialization
n = 1:Nx; f = fs/Nx*(0:Nx-1); % initialization: sample indexes, frequency values
for k=1:length(fc)                        %
    x = x + cos(2*pi*fc(k)/fs*(0:Nx-1));  % test signal generation
end
x = x + 0.1*rand(1,Nx);

% x = randn(1,Nx);                                 % alternative signal 1
% [x,fs]= audioread('bach44100.wav',[1,Nx]); x=x'; % alternative signal 2
% [x,fs]= audioread('piano.wav',[1,Nx]); x=x'; % alternative signal 2

    figure; plot(x(n)); title('x(n)'); pause
    figure; plot(f,20*log10(abs(fft(x(n))))/Nx); title('|X(f)| [dB]'); pause

% Filter bank matrices
win = 2*sqrt(M)*prototype(N);    
% prototype filter
figure; plot(win); title('w(n)'); pause   %
win = ones(M,1)*win;                      %
[n,k] = meshgrid(0:(N-1),0:(M-1));        %
C = win.*cos(pi/M*(k+1/2).*(n-M/2));      % analysis  matrix incl. filter
D = win.*cos(pi/M*(k+1/2).*(n+M/2));      % synthesis matrix incl. filter
D = D';                                   %

% Checking FB frequency response
figure;
K=5; KN=K*N; f = fs/KN*(0:KN-1);
figure; plot(f,20*log10(abs(fft(C',K*N)))); title('Bank of filters [dB]'); xlabel('f [Hz]'); grid; pause

% Analysis - synthesis
bx = zeros(1,N);                              % buffer for input samples
sb = zeros(Nmany,M);                          % matrix for subband samples  
y1 = zeros(1,Nx);                             % output 1

for m=1:Nmany                                 % MAIN LOOP
  % Analysis filter bank
    bx = [ x(m*M:-1:(m-1)*M+1) bx(1:N-M) ];   % input buffer update
    BX = C*bx';                               % filtration
    sb(m,:) = BX;                             % storing subband samples
    
  % doing something 
  % BX(num(2)) = 0;
  
  % Synthesis filter bank
    by = D*BX;                                % signal synthesis from subbands                   
    n1st = 1+(m-1)*M; nlast = N + (m-1)*M;    % sample indexes
    n = n1st : 1 : nlast;                     %
    y1( n ) = y1( n ) + by';                  % addition with overlapping
end    

    figure;
    subplot(411); plot(sb(:,num(1)));   title('sb( no1 )'); 
    subplot(412); plot(sb(:,num(1)+2)); title('sb( no1 + offs)');
    subplot(413); plot(sb(:,num(2)));   title('sb( no2 )');
    subplot(414); plot(sb(:,num(2)+2)); title('sb( no2 + offs )'); pause

% Output signal - synchronization with input
Noffs=N-M+2; n1=1:(Nx-N)-Noffs+1; n2=Noffs:(Nx-N);
xr=x(n1); yr=y1(n2);
SNR_y_dB = 10*log10( sum(xr.^2) / sum((xr-yr).^2) ), pause

    figure; plot(n1,xr,'r',n1,yr,'b'); title('INPUT (red) OUTPUT (blue)');  pause
    figure; plot(n1,xr-yr); title('Difference IN-OUT');  pause

% ###################################
% TESTING QUANTIZATION
% ###################################

% Quantization of the original signal
b=4; sc=2^(b-1)+0.49; xmax=max(abs(x)); xq = xmax * fix( sc*x/xmax ) / sc;

% Bit allocation for subbands: 2^b+1 quantization levels 
%b = [5*ones(M/4,1); 4*ones(M/4,1); 3*ones(M/4,1); 2*ones(M/4,1)]; sc=2.^(b-1)+0.49; sc=sc';
b = 4*ones(M,1); sc = 2.^(b-1)+0.49; sc=sc';

% Quantization in sub-bands
sbref = sb;                                                 % copy
figure; mesh(20*log10(abs(sb))); title('sb IN'); pause      % figure IN
sbmax = max( abs( sb(M+1:end,:) ) );                        % finding max values
figure; semilogy( sbmax ); title('MAX in subbands'); pause  % ploting them    
% work=sbmax'; save('sbmax.dat','work','-ascii');           % storing to disc
sb = sbmax .* fix( sc .* (sb./sbmax) ) ./ sc;               % sb quantization
figure; mesh(20*log10(abs(sb))); title('sb OUT'); pause     % figure OUT

% Signal synthesis from sub-bands
y2 = zeros(1,Nx);                             % output 2
for m=1:Nmany
    BX = sb(m,:)';                            % restore subband samples
    by = D*BX;                                % synthesis filter bank
    n1st = 1+(m-1)*M; nlast = N + (m-1)*M;    % indexes of signal samples
    n = n1st : 1 : nlast;                     % from - to
    y2( n ) = y2( n ) + by';                  % output signal formation
end
xqs = y2;                                     % for quatization in subbands 

% Spectra of original and quantized signals
if(0)  % our calculations
   w = hanning(Nx)'; f=fs/Nx*(0:Nx-1);
   X   = 20*log10( 2*abs(fft(w.*x))/sum(w) );
   Xq  = 20*log10( 2*abs(fft(w.*xq))/sum(w) ); 
   Xqs = 20*log10( 2*abs(fft(w.*xqs))/sum(w) );
else  % Matlab function
   [X, f]  = periodogram(  x,[],512,fs,'power'); X=10*log10(X);
   [Xq,f]  = periodogram( xq,[],512,fs,'power'); Xq=10*log10(Xq);
   [Xqs,f] = periodogram(xqs,[],512,fs,'power'); Xqs=10*log10(Xqs);
end
    figure; plot(f,X,'r-',f,Xq,'b-',f,Xqs,'g-'); xlabel('f (Hz)');
    title('Power (dB)');
    legend('x(n)','xq(n)','xqs(n)','Location','SouthWest'); grid; pause

% Comparison of the original and quantized signals (two methods)
SNR_xq_dB  = 10*log10( sum(x(n1).^2) / sum((x(n1)-xq(n1)).^2 ) ),
SNR_xqs_dB = 10*log10( sum(x(n1).^2) / sum((x(n1)-xqs(n2)).^2) ),
pause

figure; plot( n1, x(n1),'ro-', n1, xq(n1),'bx-', n1, xqs(n2),'g*-' );
title('x(n)(RED o), xq(n)(BLUE x), xqs(n)(GREEN *)');
legend('x(n)','xq(n)','xqs(n)');
pause

figure; plot( n1, x(n1)-xq(n1),'bx-', n1, x(n1)-xqs(n2),'g*-');
title('Difference IN-OUT: xq(n) (BLUE x) xqs(n) (GREEN *)');
legend('x(n)-xq(n)','x(n)-xqs(n)');
pause

sound(x,  fs); pause
sound(xq, fs); pause
sound(xqs,fs); pause
