
% lab15_ex_mp2.m

% ------------------------------------------------------------------
% Encoder and decoder of MP2 standard (fs = 32, 44.1 and 48 kHz)
% ------------------------------------------------------------------
  clear all; close all;
  
global BArow BAQuantNo Qbits Qgroup 
  
% QUANTIZATION (Q) CLASSES FOR LAYER 2
% in rows: Q levels, bits per a codeword, samples per a codeword, SNR (dB)    
% ====================================================================================================================  
% Class no     1   2   3   4     5     6     7     8     9     10    11    12    13    14    15    16    17
% ====================================================================================================================  
  Qlevels  = [ 3   5   7   9     15    31    63    127   255   511   1023  2047  4095  8191  16383 32767 65535 ];
  Qbits    = [ 5   7   3   10    4     5     6     7     8     9     10    11    12    13    14    15    16    ];
  Qgroup   = [ 3   3   1   3     1     1     1     1     1     1     1     1     1     1     1     1     1     ];
  Qsnr     = [ 7   11  16  20.84 25.28 31.59 37.75 43.84 49.89 55.93 61.96 67.98 74.01 80.03 86.05 92.01 98.01 ];
% ====================================================================================================================  

% TABLES OF BIT ALLOCATIONS (BA) for consecutive 32 subbands and bitrates: 56, 64, 80, 96, 112, 128, 160, 192 kbs
  sblimit1 = 27; sumabit1 = 88; % sblimit - number of last subband
  sblimit2 = 30; sumabit2 = 94; % sumabit - number of bits for indexes from the BA table
% ====================================================================================================================  
% Subband no   1 2 3 4 5 6 7 8 9 ..................................27....30   % for each subband
% ====================================================================================================================  
  BAbits   = [ 4 4 4 4 4 4 4 4 4 4 4 3 3 3 3 3 3 3 3 3 3 3 3 2 2 2 2 2 2 2 ]; % no of bits for Q class number
  BArow    = [ 1 1 1 2 2 2 2 2 2 2 2 3 3 3 3 3 3 3 3 3 3 3 3 4 4 4 4 4 4 4 ]; % no of reference - row in tables ABxxxx
  BAmax    = [ 15 15 7 3 ]; % last element in consecutive rows 1, 2, 3, 4, ... of  tables ABxxxx
% ====================================================================================================================  
% Column no     1   2   3      4     5     6     7      8     9     10    11    12    13    14    15
% ====================================================================================================================  
  BAQuantNo = [ 1   3   5      6     7     8     9      10    11    12    13    14    15    16    17;
                1   2   3      4     5     6     7      8     9     10    11    12    13    14    17;
                1   2   3      4     5     6     17     0     0     0     0     0     0     0     0;
                1   2   17     0     0     0     0      0     0     0     0     0     0     0     0       ];
  BApSNR    = [ 7   9   9.28   6.31  6.16  6.09  6.05   6.04  6.03  6.02  6.03 6.02   6.02  5.96  6.00;
                7   4   5      4.84  4.44  6.31  6.16   6.09  6.05  6.40  6.03 6.02   6.03  6.02  17.98;
                7   4   5      4.84  4.44  6.31  66.42  0     0     0     0     0     0     0     0;
                7   4   87.01  0     0     0     0      0     0     0     0     0     0     0     0       ];
  BApbit    = [ 60  48  36     36    36    36    36     36    36    36    36    36    36    36    36;
                60  24  24     12    24    36    36     36    36    36    36    36    36    36    108;
                60  24  24     12    24    36    396    0     0     0     0     0     0     0     0;
                60  24  492    0     0     0     0      0     0     0     0     0     0     0     0       ];
% ===================================================================================================================  

% Quantuzation coefficients  
  a = [  0.750000000    0.625000000    0.875000000    0.562500000    0.937500000 ...
         0.968750000    0.984375000    0.992187500    0.996093750    0.998046875 ...
         0.999023438    0.999511719    0.999755859    0.999877930    0.999938965 ...
         0.999969482    0.999984741    ];
  b = [ -0.250000000   -0.375000000   -0.125000000   -0.437500000   -0.062500000 ...
        -0.031250000   -0.015625000   -0.007812500   -0.003906250   -0.001953125 ...
        -0.000976563   -0.000488281   -0.000244141   -0.000122070   -0.000061035 ...
        -0.000030518   -0.000015259  ];
  
% Re-quantization coefficients  
  c = [	  1.33333333333  1.60000000000  1.14285714286  1.77777777777  1.06666666666 ...
	      1.03225806452  1.01587301587  1.00787401575  1.00392156863  1.00195694716 ...
	      1.00097751711  1.00048851979  1.00024420024  1.00012208522  1.00006103888 ...
	      1.00003051851  1.00001525902  ];
  d = [   0.50000000000  0.50000000000 	0.25000000000  0.50000000000  0.12500000000 ...
	      0.06250000000	 0.03125000000  0.01562500000  0.00781250000  0.00390625000 ...
	      0.00195312500  0.00097656250  0.00048828125  0.00024414063  0.00012207031 ...
          0.00006103516  0.00003051758  ];
     
% Scaling coefficients
  sc = [  2.00000000000000  1.58740105196820    1.25992104989487    1.00000000000000    0.79370052598410 ...
	      0.62996052494744 	0.50000000000000	0.39685026299205	0.31498026247372	0.25000000000000 ...
	      0.19842513149602  0.15749013123686	0.12500000000000	0.09921256574801	0.07874506561843 ...
	      0.06250000000000	0.04960628287401	0.03937253280921	0.03125000000000	0.02480314143700 ...
	      0.01968626640461	0.01562500000000	0.01240157071850	0.00984313320230	0.00781250000000 ...
	      0.00620078535925	0.00492156660115	0.00390625000000	0.00310039267963	0.00246078330058 ...
	      0.00195312500000	0.00155019633981	0.00123039165029	0.00097656250000	0.00077509816991 ...
	      0.00061519582514	0.00048828125000	0.00038754908495	0.00030759791257	0.00024414062500 ...
	      0.00019377454248	0.00015379895629	0.00012207031250	0.00009688727124	0.00007689947814 ...
	      0.00006103515625  0.00004844363562 	0.00003844973907 	0.00003051757813 	0.00002422181781 ...
	      0.00001922486954	0.00001525878906	0.00001211090890	0.00000961243477	0.00000762939453 ...
	      0.00000605545445	0.00000480621738	0.00000381469727	0.00000302772723	0.00000240310869 ...
          0.00000190734863	0.00000151386361	0.00000120155435  10^-20 ];
  sctab = [ 32 16 8 4 2 1 ];  
  
% Tables for chosing of transmitted scalefactors (sc)
  scCode = [ 0 3 3 3 0; 1 2 2 2 1; 2 2 2 2 1; 2 2 2 2 0; 0 3 3 3 0 ]; % code: 0=3sc, 1=2sc, 2=1sc; 3=2sc
  scNum  = [ 0 2 2 3 0; 1 1 1 4 1; 1 1 1 3 1; 2 2 2 3 0; 0 2 2 3 0 ]; % which coeffs is repeating
  scBit = [ 20 14 8 14 8 ]; % number of bits for sc numbers and info about them
  
% Groupnig parameters (in row: bits/sample, basis1, basis2, basis3, bits/triplet)
  group = [ 2  1  3  9   5;
            3  1  5  25  7;
            4  1  9  81  10 ]; 
         
% ===================================================================================================================  
% ===================================================================================================================  
% ===================================================================================================================  

% Main parameters
  M=32;           % number of sub-bands (cosine channels)
  L=512;          % low-pass prototype filter length
  K=12;           % 1/3 of all samples in each sub-band in one audio frame
  fs = 44100;     % sampling frequency: 32000, 44100 or 48 kHz
  layer = 2;      % coding layer: only 2 is available
  bitrate = 64;  % 56, 64, 80, 96, 112, 128, 160, 192 kbits/s per channel
  show = 0;       % 0/1 show temporary results of calculations
  
  MM=2*M;                   % number of complex-value channels, decimation order
  Lp=L/MM;                  % length of polyphase sequences
  M2=2*M; M3=3*M; M4=4*M;   % auxiliary parameters
  K2=2*K; K3=3*K; block1=1:K; block2=K+1:2*K; block3=2*K+1:3*K;
  j = 0 : Lp-1;
  
% Generate or read from disc a testing signal - add yours!
% Nx=5*1152; n=0:Nx-1; x=sin(2*pi*n/27)+sin(2*pi*n/49)+sin(2*pi*n/155); x=x'/max(x);
% Nx=5*1152; x=0.3*randn(1,Nx)';
% [x,fs]= audioread('a44.wav',[1,3*fs]);
  [x,fs]= audioread('bach44100.wav',[1,3*fs]);
% [x,fs]= audioread('Badu.wav',[1, 5*fs]); x=0.75*x(:,1);  
% [x,fs]= audioread('salvation.wav',[1, 5*fs]); x=x(:,1);  
% [x,fs]= audioread('loussier1.wav',[1, 5*fs]); x=x(:,1);  
% [x,fs]= audioread('glockenspiel.wav',[1, 5*fs]); x=x(:,1);  
% [x,fs]= audioread('track1.wav',[1, 5*fs]); x=x(:,1);  
% size(x), fs, pause
  
  plot(x); title('Signal for coding'); pause % show the signal
  soundsc(x,fs);                             % play it using laudspeakes or headphones
  Np=M*3*K;                                  % total number of audio samples in one data frame   
  Nx=length(x); x=x';                        % length of the test signal
  Nr=ceil(Nx/Np); x=[x zeros(1,Nr*Np-Nx)];   % appending zeros to the signal end
  
% Parameters of bit allocation (Fbits - number of bits per frame without prefix and side (alloc) info)
  sblimit = sblimit1; sumabit = sumabit1; 
  if ((fs==32000 || fs==48000) && (bitrate>80)) sblimit=sblimit2; sumabit = sumabit2; end
  Fbytes = floor(144*1000*bitrate/fs);   % 1000 because bitrate in kbs
  Fbits = 8*Fbytes - 32 - sumabit;       % subract prefix length and side (alloc) inf
  Bbitscopy = Fbits;
  
% Load original MP2 impulse response (window) p(n) of low-pass prototype filter
  load enwindow.dat; pe1=enwindow';  % standard encoder window
  load dewindow.dat; pd1=dewindow';  % standard decoder window
  Nw = length(pe1); n = 1:Nw;        % ich dugo
  figure; plot(n,pe1,'r',n,pd1,'b'); title('imp response: encoder (RED) decoder(BLUE)'), pause
% Generate pseudo-MP2 impulse response  
  p = prototype(L); p = sqrt(M)*p; figure; plot(p); title('Prototype filter'); grid; pause % generated window
  for m = 1 : 2 : Lp-1
     p( m*MM+1 : m*MM+MM ) = -p( m*MM+1 : m*MM+MM ); % negation after 64 samples
  end
  pe2 = p/(2*sqrt(2)); pd2 = (4*sqrt(2))*p;          % new windows for encoder and decoder  
  figure; plot(n,pe1,'r',n,pe2,'b'); title('encoder window:  norm (RED) our (BLUE)'), pause
  figure; plot(n,pd1,'r',n,pd2,'b'); title('decoder window:  norm (RED) our (BLUE)'), pause
% Choosing impulse response from MP2 (1) or ours (2)   
  pe = pe1; pd = pd1;   % 1 or 2 
  
% Polyphase matrices for signal analysis (A) and synthesis (B)
  m=0:2*M-1;
  for k=0:M-1
     A(k+1,1:2*M) = 2*cos((pi/M)*(k+0.5).*(m-M/2));  % analysis matrix  %
     B(k+1,1:2*M) = 2*cos((pi/M)*(k+0.5).*(m+M/2));  % synthesis matrix %
  end
  
% Initialization of signal bufferes  
  bx512 = zeros(1,L); bx2304 = zeros(1,2*3*K*M); bv=zeros(1,2*L); y=[]; 

% #####################################################################################
% MAIN LOOP: MP2 encoder and decoder (one by one, frame after frame)
% #####################################################################################

  
  for iter = 1 : Nr % next frame
     
      iter
     
    % Take new data frame into the buffer (36 * 32 subbands = 1152 samples)
      bx1152 = x( 1+(iter-1)*Np : Np + (iter-1)*Np );
      bx2304 = [ bx1152 bx2304(1:3*K*M) ];
      
    % KODER ######################################################## 
    
    % Psycho-acoustical model: calculate FFT and SMR (Signal to Mask Ratio)
      [SMR ] = MP2psycho( bx2304, fs, show);
    % figure; plot(SMR); title('Signal-to-Mask Ratio'); grid; pause
      SMR(sblimit+1:M) = -300;
      
    % Analysis filter bank  
      sb = [];
      for k = 1 : K3
        % loadind 32 new samples into the buffer
          bx512 = [ bx1152(k*M:-1:(k-1)*M+1) bx512(1:L-M) ];
        % bank of M=32 filters working in parallel
          for m = 1 : 2*M
              u(m) = sum( bx512(m:2*M:L).*pe(m:2*M:L) );  % polyphase filtration
          end
          sb32 = A*u';                                    % cosine modulation
          sb = [sb; sb32'];                               % add as a new row   
      end
      if(show==1) min(min(sb)), max(max(sb)), pause; end
      
      ssb = sb; % if we are testing only filter bank, below part is skipped
      
if(1) % 0 = test only filter bank, skip the rest, 1 = continue       
      
    % Find all scaling coefficients for each subband (3 groups with 12 samples)
      sbmax(1,1:M) = max(abs(sb(1:K,:))); sbmax(2,1:M) = max(abs(sb(K+1:2*K,:))); sbmax(3,1:M) = max(abs(sb(2*K+1:3*K,:)));
      for k = 1 : M
         for l = 1 : 3
            tmp = find(sc>sbmax(l,k)); scmax(l,k) = tmp(end);  % if error here, set x=x/2
            sbm = sbmax(l,k); indx = 0;                            % the same as above, but slower
            for m = 1 : log2(64)                                   %
               indx = indx + sctab(m);                             %
               if( sbm > sc( indx ) ) indx = indx - sctab(m); end  %
            end                                                    %
            scmax(l,k)=indx;
         end
      end  
    % sb, sbmax(1:3,1:M),pause
    % sbmax(1:3,1:M), scmax(1:3,1:M), pause
      if(show==1) scmax(1:3,1:M), pause; end
      
    % Choose scaling coefficients, info about them will be stored/transmitted
      m=1:M; dsc1(m)=scmax(1,m)-scmax(2,m); dsc2(m)=scmax(2,m)-scmax(3,m); Fbitssci=[];
      csc1( find( dsc1 <= -3       ) ) = 1;   csc2( find( dsc2 <= -3       ) ) = 1;
      csc1( find( -3<dsc1 & dsc1<0 ) ) = 2;   csc2( find( -3<dsc2 & dsc2<0 ) ) = 2;
      csc1( find( dsc1 == 0        ) ) = 3;   csc2( find( dsc2 == 0        ) ) = 3;
      csc1( find( 0<dsc1 & dsc1<3  ) ) = 4;   csc2( find( 0<dsc2 & dsc2<3  ) ) = 4;
      csc1( find( 3 <= dsc1        ) ) = 5;   csc2( find( 3 <= dsc2        ) ) = 5;
      
      for k = 1 : sblimit
          kod = scCode(csc1(k),csc2(k));  % code of sc and sci (info), from 0 to 3
          Fbitssci(k) = scBit(kod+1);     % number of bits for sc and sci (info) 
      end
    % dsc1, csc1, pause
    % dsc2, csc2, pause
      
    % Do adaptive bit alocation for each sub-band using 'water filling" technique
      Fbits = Bbitscopy; Fballoc = zeros(1,sblimit); SMRcopy=SMR(1:sblimit);
      while( (Fbits>0) )
       % stem(SMRcopy); pause
       % Fbits, pause
         k = find( SMRcopy == max(SMRcopy) ); 
         if( Fballoc(k) == 0 ) Fbits = Fbits - Fbitssci(k); end
         if(Fbits < 0) break; end
      %  k, Fbitssci(k), Fbits, pause
         
         Fballoc(k) = Fballoc(k)+1;
         extrabits = BApbit( BArow(k), Fballoc(k) );
         Fbits = Fbits - extrabits;
         if(Fbits < 0) Fballoc(k) = Fballoc(k)-1; break; end
         SMRcopy(k) = SMRcopy(k) - BApSNR( BArow(k), Fballoc(k) );
       % k, Fballoc(k), ekstrabity, Fbits, pause
      end
      if(show==1) Fballoc, pause; end
      
    % Normalize sub-band samples in transmitted sub-bands
      nsci = 1; nsc=1; Fscfi = []; Fscf = [];
      for k = 1 : sblimit
         if(Fballoc(k)>0)
            code = scCode(csc1(k),csc2(k));  % code for sc and sci, from 0 to 3
            nr  =  scNum(csc1(k),csc2(k));   % number of sc which is repeated
            
            Fscfi(nsci) = code; nsci=nsci+1;  % store
            if(code==0) msc1=scmax(1,k); msc2=scmax(2,k); msc3=scmax(3,k); 
                        Fscf(nsc)=msc1; Fscf(nsc+1)=msc2; Fscf(nsc+2)=msc3; nsc=nsc+3; end
            if(code==1) msc1=scmax(1,k); msc2=msc1; msc3=scmax(3,k); 
                        Fscf(nsc)=msc1; Fscf(nsc+1)=msc3; nsc=nsc+2; end
            if(code==2) if(nr~=4) msc=scmax(nr,k); else msc=min(scmax(1:3,k)); end
                        msc1=msc; msc2=msc; msc3=msc; Fscf(nsc)=msc; nsc=nsc+1; end      
            if(code==3) msc1=scmax(1,k); msc2=scmax(nr,k); msc3=msc2;
                        Fscf(nsc)=msc1; Fscf(nsc+1)=msc2; nsc=nsc+2; end
            sbn(1:K,    k) = sb(1:K,    k)./sc(msc1);
            sbn(K+1:K2, k) = sb(K+1:K2, k)./sc(msc2);
            sbn(K2+1:K3,k) = sb(K2+1:K3,k)./sc(msc3);
         end   
      end
      if(show==1) Fscfi, pause; end
      if(show==1) min(min(sbn)), max(max(sbn)), pause; end
      
      ssbk = sbn; % if we skip sample quantization and grouping
      
if(1) % 0 = test with normalization but without quantization and grouping, 1 = do everything 

    % Quantize audio samples in transmitted sub-bands 
      Fsb=[]; ns = 0; sblevels = zeros(1,sblimit); sbbits= zeros(1,sblimit);
      
      for m = 1 : K   % K=12 times
          m3 = 1+(m-1)*3 : 3+(m-1)*3;
          for k = 1 : sblimit  % together 3 samples at once in one subband
              index = Fballoc(k); 
              if( index > 0 )
                 Qnum  = BAQuantNo( BArow(k), index ); 
                 Qlevs = Qlevels( Qnum ); sblevels(k) = Qlevs; sbbits(k) = Qbits( Qnum );
         
                 ak = a( Qnum );
                 bk = b( Qnum );
            
                 if( Qlevs==3 || Qlevs==5 || Qlevs==9 ) scale = (Qlevs-1);
                 else scale = (Qlevs+1)/2; end
            
                 sbk(1:3) = floor( (ak * sbn(m3,k) + bk)*scale + scale );
               % if (min(sbk) < 0) k, Qlevs, sbk, pause; end % DEBUG
               
                 if(Qlevs==3) sbk = sbk(1) + 3.*sbk(2) +  9.*sbk(3); end
                 if(Qlevs==5) sbk = sbk(1) + 5.*sbk(2) + 25.*sbk(3); end
                 if(Qlevs==9) sbk = sbk(1) + 9.*sbk(2) + 81.*sbk(3); end
               % if (min(sbk) < 0) k, index, sbk, pause; end  % DEBUG
            
                 Fsb = [ Fsb; sbk(:) ];
               % sbk(:), pause
             end   
         end       
      end
    % stem(sbbits); pause
    % sblevels, sbbits, pause  
    % Fsb = abs(Fsb);
    
    % Buit an MP2 binary frame: header, info about bit allocation and scale factors used,
    % number of scalefactors, quantized samples
    
    % Write to disc frame of compressed audio
      MP2write( Fballoc, Fscfi, Fscf, Fsb, Fbytes, sblevels, sbbits, fs, bitrate, sblimit, show );
      
    % Read from disc the same frame of compressed audio
     [ FballocX, FscfiX, FscfX, FsbX, fs, bitrate, sblimit ] = MP2read(show, Fbytes);
    % Optional copy  
      Fballoc = FballocX;
      Fscfi = FscfiX;
      Fscf = FscfX;
      Fballoc = FballocX;
      
    % DEKODER ##################################################################################
    
    % Synchronize with frame beginning usung a unique SYNCHRO pattern, decode the header,
    % head information about bit allocation and scale factors
    
    % Re-quantization of signal samples
      ns = 0; ssbk=[];
      
      for m = 1 : K % K=12 razy
          m1 = 1+(m-1)*3;  m3 = 1+(m-1)*3 : 3+(m-1)*3;
          for k = 1 : sblimit % % together 3 samples at once in one subband
              indeks = Fballoc(k);
              if( indeks > 0 )
                 Qnum = BAQuantNo( BArow(k), indeks );
                 layery = Qlevels( Qnum );
                 ck = c( Qnum );
                 dk = d( Qnum );
                 if(layery==3 || layery==5 || layery==9)
                    scale = (layery-1);
                    tmp = Fsb(ns+1); ns=ns+1;
                    for(n=0:2) ssbk(m1+n,k)=rem(tmp,layery); tmp = floor(tmp/layery); end
                 else
                    scale = (layery+1)/2;
                    ssbk(m3,k) = Fsb(ns+(1:3)); ns=ns+3;
                 end
                 ssbk(m3,k) = ck*( (ssbk(m3,k)-scale) / scale + dk);
              end   
          end  
      end
      
end % end of skipping the gouping and quantization
      
    % De-normalization of samples
      nsci = 1; nsc=1;
      for k = 1 : sblimit
         if(Fballoc(k)>0)
            code = Fscfi(nsci); nsci=nsci+1;  % read code about sc and sci
            if(code==0) msc1=Fscf(nsc); msc2=Fscf(nsc+1); msc3=Fscf(nsc+2); nsc=nsc+3; end
            if(code==1) msc1=Fscf(nsc); msc2=msc1; msc3=Fscf(nsc+1); nsc=nsc+2; end
            if(code==2) msc1=Fscf(nsc); msc2=msc1; msc3=msc1; nsc=nsc+1; end  
            if(code==3) msc1=Fscf(nsc); msc2=Fscf(nsc+1); msc3=msc2; nsc=nsc+2; end
            ssb(block1, k) = sc(msc1) *  ssbk(block1, k);
            ssb(block2, k) = sc(msc2) *  ssbk(block2, k);
            ssb(block3, k) = sc(msc3) *  ssbk(block3, k);
         else
            ssb(1:K3,k) = zeros(K3,1);
         end
         if(show==1) work = [ssb(1:K3,k) sb(1:K3,k)], pause; end
      end
      ssb(1:K3,sblimit+1:M) = zeros(K3,M-sblimit);

end % end of skipping everytning except the filter banks

    % Audiou synthesis filter bank
      for k = 1 : K3
         v = B'*ssb(k,1:M)';                         % cosine demodulation
         bv = [ v' bv(1:2*L-MM) ];                   % putting into buffer
         for n = 1 : M
            ys(n) = sum( bv(n+M4*j).*pd(n+M2*j) ) + sum( bv(n+M3+M4*j).*pd(n+M+M2*j) );
         end
         y = [ y ys ];
      end
      
% #####################################################################################
  end % end of proceesing the next frame
% #####################################################################################
  
% Output de-compressed audio signal
  y = y/2; % if all steps have been done, then amplify 2-times
  Noffs=L-M+2; n1=1:Nx-Noffs+1; n2=Noffs:Nx;
  xr=x(n1); yr=y(n2);
  figure; plot(n1,xr,'r',n1,yr,'b'); title('IN (red) OUT (blue)');  pause
  figure; plot(n1,xr-yr); title('Difference IN - OUT');  pause
  SNR_dB = 10*log10( sum(xr.^2) / sum((xr-yr).^2) ), pause
% Test odsuchowy  
  figure; plot(y); title('De-compressed signal'); pause
  soundsc(y, fs)
  
  b = 6;
  yq = fix(2^b*xr) / 2^b;
  Nxr = length(xr); f=fs/Nxr*(0:Nxr-1); w = hanning(Nxr)';
  if(0)
    Xr = 20*log10( 2*abs(fft(w.*xr))/sum(w) );
    Yr = 20*log10( 2*abs(fft(w.*yr))/sum(w) ); 
    Yq = 20*log10( 2*abs(fft(w.*yq))/sum(w) );
  else
    [Xr,f]=periodogram(xr,[],512,fs,'onesided'); Xr=10*log10(Xr);
    [Yr,f]=periodogram(yr,[],512,fs,'onesided'); Yr=10*log10(Yr);
    [Yq,f]=periodogram(yq,[],512,fs,'onesided'); Yq=10*log10(Yq);
  end
  plot(f,Xr,'r-',f,Yr,'b-',f,Yq,'g-'); xlabel('f (Hz)');
  title('Power/frequency (dB/Hz): original (RED), mp2 (BLUE), quant (GREEN)'); grid; pause
 